@@ -71,7 +71,7 @@ static char * kanamap[128]=
  * Note that "long enough" should probably be
  *   kstrlen(kstring) *2 * 2;
  *
- * SIgh. better put a limit on it: MAXROMAJI
+ * Sigh. better put a limit on it: MAXROMAJI
  *
  * Note that we also have to special-case punctuation. sigh.
  *  "(",     ")"
@@ -79,6 +79,10 @@ static char * kanamap[128]=
  *
  * All this, is so we can print out the "reading" entries, for folks
  * who can't read kana. WIMPS! :-)
+ *
+ * Currently, this is a state engine.
+ * It would be much more consistent if I just used romajimap1/2[] and worked
+ * backwards. Oh well.
  */
 void kanatoromaji(XChar2b *kstring, char *estring)
 {
@@ -252,9 +256,14 @@ static char *romajimap1[]=
 	"sha","shi", "shu","sho",
 	"cha","chu","cho",
 	"gya","gyu","gyo",
-	/*"mya","myu","myo",
+
+	"hya","hyu","hyo",
+	"bya","byu","byo",
+	"pya","pyu","pyo",
+
+	"mya","myu","myo",
 	"nya","nyu","nyo",
-	*/
+
 	"kya","kyu","kyo",
 	"rya","ryu","ryo",
 	"chi","tsu",
@@ -276,10 +285,16 @@ static char *romajimap1[]=
 	"ra","ri","ru","re","ro",
 	"wa","wo",
 	"ya","yu","yo",
+	"_ya","_yu","_yo", /* Small letters, the hard way. For romaji input*/
 	"a","i","u","e","o",
-	"-", "'","(",")",
+	
+	"-", "'","(",")", /* These are here not so much for romaji input,
+	                   * as to match dictionary search patterns
+			   */
 	""
-	/* See also hardcodes for special keys in Handle_romajikana */
+	/* See also hardcodes for special keys in Handle_romajikana()
+	 * and matchromaji()
+	 */
 };
 /* must match order of romajimap1[]!!*/
 static XChar2b romajimap2[150][3]=
@@ -310,6 +325,26 @@ static XChar2b romajimap2[150][3]=
 	{{0x24, 0x2e}, {0x24, 0x65}, {0, 0}},/*gyu*/
 	{{0x24, 0x2e}, {0x24, 0x67}, {0, 0}},/*gyo*/
 
+	{{0x24, 0x52}, {0x24, 0x63}, {0, 0}},/*hya*/
+	{{0x24, 0x52}, {0x24, 0x65}, {0, 0}},/*hyu*/
+	{{0x24, 0x52}, {0x24, 0x67}, {0, 0}},/*hyo*/
+
+	{{0x24, 0x53}, {0x24, 0x63}, {0, 0}},/*bya*/
+	{{0x24, 0x53}, {0x24, 0x65}, {0, 0}},/*byu*/
+	{{0x24, 0x53}, {0x24, 0x67}, {0, 0}},/*byo*/
+
+	{{0x24, 0x54}, {0x24, 0x63}, {0, 0}},/*pya*/
+	{{0x24, 0x54}, {0x24, 0x65}, {0, 0}},/*pyu*/
+	{{0x24, 0x54}, {0x24, 0x67}, {0, 0}},/*pyo*/
+
+	{{0x24, 0x5f}, {0x24, 0x63}, {0, 0}},/*mya*/
+	{{0x24, 0x5f}, {0x24, 0x65}, {0, 0}},/*myu*/
+	{{0x24, 0x5f}, {0x24, 0x67}, {0, 0}},/*myo*/
+
+	{{0x24, 0x4b}, {0x24, 0x63}, {0, 0}},/*nya*/
+	{{0x24, 0x4b}, {0x24, 0x65}, {0, 0}},/*nyu*/
+	{{0x24, 0x4b}, {0x24, 0x67}, {0, 0}},/*nyo*/
+
 	{{0x24, 0x2d}, {0x24, 0x63}, {0, 0}},/*kya*/
 	{{0x24, 0x2d}, {0x24, 0x65}, {0, 0}},/*kyu*/
 	{{0x24, 0x2d}, {0x24, 0x67}, {0, 0}},/*kyo*/
@@ -411,6 +446,10 @@ static XChar2b romajimap2[150][3]=
 	{{0x24, 0x66}, {0, 0}},/*yu*/
 	{{0x24, 0x68}, {0, 0}},/*yo*/
 
+	{{0x24, 0x63}, {0, 0}},/*_ya*/
+	{{0x24, 0x65}, {0, 0}},/*_yu*/
+	{{0x24, 0x67}, {0, 0}},/*_yo*/
+
 	{{0x24, 0x22}, {0, 0}},/*a*/
 	{{0x24, 0x24}, {0, 0}},/*i*/
 	{{0x24, 0x26}, {0, 0}},/*u*/
@@ -423,12 +462,12 @@ static XChar2b romajimap2[150][3]=
 	{{0x21, 0x4a}, {0, 0}},/*"(" for open-paren. */
 	{{0x21, 0x4b}, {0, 0}},/*")" for close-paren. */
 
-	{0}
+	{{0}}
 	
 };
 
 
-/* util for romajitokana
+/* suborutine for romajitokana()
  * return 1 if regular char* can be said to match kstring,
  * Like strstr, not like strcmp
  */
@@ -436,10 +475,21 @@ static int matchromaji(char *romaji, XChar2b *kstring)
 {
 	while(*romaji!='\0')
 	{
-		if(kstring->byte1 != 0x23)
+		if(kstring->byte1 > 0x23){
 			return 0;
-		if(*romaji != kstring->byte2)
+		}
+		/*durn non-ascii punctuation codes...*/
+		if(kstring->byte1==0x21){
+			switch(kstring->byte2){
+				case 0x32: /* kana underscore */
+				if(*romaji != '_'){
+					return 0;
+				}
+			}
+		} else {
+			if(*romaji != kstring->byte2)
 			return 0;
+		}
 		romaji++;
 		kstring++;
 	}
@@ -461,13 +511,11 @@ void romajitokana(XChar2b *kstart)
 	XChar2b *kparse;
 	int matchcount;
 
-	int stoploop=0;/* used by JUMBO-SWITCH, lower down */
-				
-	
-	/* start off by looking for pseudo-8-bit chars, aka
-	 * romaji
+	/* start off by looking for pseudo-8-bit chars, aka romaji
+	 * Allow for JIS-symbol equivalent of ASCII punctuation, at
+	 * 0x21##, as well as pseudo-ascii at 0x23##
 	 */
-	while(kstart->byte1 != 0x23)
+	while(kstart->byte1 > 0x23)
 	{
 		if(kstart->byte1 == 0)
 		{
@@ -502,7 +550,8 @@ void romajitokana(XChar2b *kstart)
 
 	if((kstart[0].byte1==0x23) &&
 	   (kstart[0].byte2=='n'))
-		if(kstart[1].byte1!=0)
+		if((kstart[1].byte1!=0) &&
+		   (kstart[1].byte2!='y'))
 		{
 			kstart[0].byte1=0x24;
 			kstart[0].byte2=0x73;
@@ -581,15 +630,23 @@ void Handle_romajikana(Widget w, XtPointer closure, XEvent *e, Boolean *cont)
 	   case XK_minus:
 		/* add "-", but in kana range
 		 * (for kanakana vowel extension )
-		 * Note that this "char" is invisible.
+		 * Note that this "char" is invisible in JIS space.
 		 */
 		addchar.byte1 = 0x23;
 		addchar.byte2 = '-';	/* 0x2d */
 		process_kinput(addchar);
 		return;
+	   case XK_underscore:
+		/* nasty nasty hack that has to be matched in
+		 * matchromaji()
+		 */
+		addchar.byte1 = 0x21;
+		addchar.byte2 = 0x32; /* 0x32='_' in JIS */
+		process_kinput(addchar);
+		return;
 	   case XK_apostrophe:
 		/* hack to get "small tsu" to work.
-		 * Note that this "char" is invisible.
+		 * Note that this "char" is invisible in JIS space
 		 */
 		addchar.byte1 = 0x23;
 		addchar.byte2 = '\'';	/* 0x3e*/
@@ -597,19 +654,23 @@ void Handle_romajikana(Widget w, XtPointer closure, XEvent *e, Boolean *cont)
 		return;
 
 	   case XK_parenleft:
-		/* hack to get "small tsu" to work.
-		 * Note that this "char" is invisible.
+		/*
+		 * Note that this "char" is invisible in JIS space
+		 * It is pseudo-ASCII
 		 */
 		addchar.byte1 = 0x23;
 		addchar.byte2 = '(';	/* 0x3e*/
+			/* in JIS it would be 214a */
 		process_kinput(addchar);
 		return;
 	   case XK_parenright:
-		/* hack to get "small tsu" to work.
-		 * Note that this "char" is invisible.
+		/*
+		 * Note that this "char" is invisible in JIS space
+		 * It is pseudo-ASCII
 		 */
 		addchar.byte1 = 0x23;
 		addchar.byte2 = ')';	/* 0x3e*/
+			/* in JIS it would be 214b */
 		process_kinput(addchar);
 		return;
 		
