@@ -1,11 +1,8 @@
 /* frequency.c
  * This file deals with frequency restrictions.
  * It unfortunately also contains some convenience routines like
- *	GetWidgetNumberval  ; read numeric value from input widget
  *	SetWidgetNumberval  ; SET display value of input widget
  *	SetWidgetHexval     ; SET display value of input widget hex.
- *	FindIndex	    ; Given special ascii code, find
- *			    ;	kanji index we need
  */
 
 #include <stdio.h>
@@ -21,6 +18,7 @@
 #include "defs.h"
 #include "externs.h"
 #include "game.h"
+#include "utils.h"
 #include "options.h"
 #include "widgets.h"
 
@@ -117,33 +115,6 @@ void SetWidgetHexval(Widget w,int val){
 }
 
 
-/* GetWidgetNumberval
- *	Get the asciiText from an input widget.
- *	Attempt to look up kanji index
- *	Set asciiText label to null if not found, or
- *	a clean decimal print of index otherwise.
- *
- *	Calls FindIndex, which accepts special prefixes.
- */
-int GetWidgetNumberval(Widget w){
-	String str;
-	int retval=0;
-
-
-	XtVaGetValues(w,XtNstring,&str,NULL);
-
-	if(w == currentkanjiNum){
-		retval = xtoi(str);
-		SetWidgetHexval(w,retval);
-	} else {
-		retval = FindIndex(str);
-		SetWidgetNumberval(w,retval);
-	}
-
-	return (retval);
-}
-
-
 /* UpdateFrequency()
  *	Change frequency restrictions on range of kanji used
  */
@@ -164,9 +135,9 @@ UpdateFrequency(Widget w,XEvent *event,String *params,Cardinal *num_parags){
 	*setthisvalue = GetWidgetNumberval(w);
 
 	if(w == frequencyHigh){
-		SetXtNumber("highfrequency", *setthisvalue);
+		SetXtrmNumber("highfrequency", *setthisvalue);
 	} else {
-		SetXtNumber("lowfrequency", *setthisvalue);
+		SetXtrmNumber("lowfrequency", *setthisvalue);
 	}
 
 	CountKanji();
