@@ -24,6 +24,9 @@
 #include "game.h"
 #include "convert.h"
 #include "badguess.h"
+#include "log.h"
+
+#include "timeout.h"
 
 TRANSLATION lastpicked=NULL;/* we can't "really" auto-initialize this...
 		  * set to zero so we can set it
@@ -133,7 +136,8 @@ Boolean UseThisKanji(int kanjinum)
  *  whether or not a kanji is active this second or not.
 
  */
-void CountKanji(){ 
+void CountKanji()
+{
 	TRANSLATION ktrans;
 	int knum;
 	numberofkanji=0;
@@ -168,19 +172,36 @@ void TallyWrong()
 	SetWidgetNumberval(kanjiMissed, numberincorrect);
 }
 
+/* User wants us to forget our missed kanji count. */
+void ClearMissed()
+{
+	int kcount;
+	for(kcount=lowestkanji; kcount<=highestkanji; kcount++)
+	{
+		if(translations[kcount]==NULL)
+			continue;
+		translations[kcount]->incorrect=0;
+	}
+	numberincorrect=0;
+	SetWidgetNumberval(kanjiMissed, numberincorrect);
+}
+
 /* pickkanji:
  *   picks an acceptably random kanji index.
  *	NOTE THAT THIS DEPENDS ON "numberofkanji" being correct!
  */
 
-TRANSLATION pickkanji(){
+TRANSLATION pickkanji()
+{
 	int rand_kanji,count;
 	
 
 	rand_kanji = lrand48()%numberofkanji;
 
-	for(count=lowestkanji;count<=highestkanji;count++) {
-		if(UseThisKanji(count)) {
+	for(count=lowestkanji;count<=highestkanji;count++)
+	{
+		if(UseThisKanji(count))
+		{
 		      rand_kanji--;
 		      if(rand_kanji <0)
 			  return translations[count];
@@ -211,7 +232,8 @@ TRANSLATION pickincorrect()
 	TRANSLATION currentpick=lastpicked;
 	int count;
 
-	if(numberincorrect==0){
+	if(numberincorrect==0)
+	{
 		puts("Internal Error.. pickincorrect called when no incorrect stored");
 		return pickkanji();
 	}
@@ -246,29 +268,36 @@ TRANSLATION pickincorrect()
 /* picktruevalue()
  *	Similar to pickkanji().
  *	This calls pickkanji() to generate random kanji when
- *	desired. 
+ *	desired, or the "next" kanji after 'lastpicked'
+ *
  *	Also tries to go back to missed kanji, ONLY when
  *	"random" order is in effect.
  *
- *	Tries to make sure same kanji isn't called in a row.
+ *	Tries to make sure same kanji isn't called in a row,
+ *	based on "lastpicked"
  *
- *	"lastpicked" is used by the "Back" command, outside this file.
- *	It is also SET somewhere else.
+ *	"lastpicked is SET somewhere ELSE, presumably just before calling
+ *	this routine. It is also used by the "Back" command
  */
-TRANSLATION picktruevalue(){
+TRANSLATION picktruevalue()
+{
 	int currentpick;
 	TRANSLATION ret_val = lastpicked;
 
-	if(!showinorder){
+	if(!showinorder)
+	{
 		/* Should we repeat a kanji they missed earlier? */
 		/* First check if they have missed any */
-		if(numberincorrect>0){
-			if(lrand48()%REPEATFRACTION == 0){
+		if(numberincorrect>0)
+		{
+			if(lrand48()%REPEATFRACTION == 0)
+			{
 				/* repeat what user missed  */
 				/*ret_val = pickincorrect();*/
 				ret_val=GetRepeat();
 			}
-			else do {
+			else
+			do {
 				ret_val = pickkanji();
 			} while (ret_val == lastpicked);
 			
@@ -276,7 +305,8 @@ TRANSLATION picktruevalue(){
 		/* This also catches the case where we repeat one
 		 * they missed, and it is the current one showing
 		 */
-		while (ret_val == lastpicked){
+		while (ret_val == lastpicked)
+		{
 			ret_val = pickkanji();
 		}
 
@@ -317,7 +347,8 @@ TRANSLATION picktruevalue(){
  *
  * Called by guessvalue()
  */
-void markasmissed(TRANSLATION kanjiP){
+void markasmissed(TRANSLATION kanjiP)
+{
 	if(kanjiP->incorrect ==0)
 		numberincorrect+=1;
 	kanjiP->incorrect += REPEATTIMES;
@@ -338,14 +369,16 @@ void markasmissed(TRANSLATION kanjiP){
 */
 /* note that "-1" on guess means "you lose: ran out of time" */
 
-void guessvalue(int guess){
+void guessvalue(int guess)
+{
 	TRANSLATION kanjiP;
 
 	kanjiP = values[guess];/* for setting "incorrect" flag */
 
 	if(guess == truevalue){
 		setstatus("Correct! Now try THIS one...");
-		switch(kanjiP->incorrect){
+		switch(kanjiP->incorrect)
+		{
 			case 0:
 				break;
 			case 1:
@@ -370,7 +403,8 @@ void guessvalue(int guess){
 	 */
 	Beep();
 	setstatus("Incorrect.");
-	if(guess != -1) {
+	if(guess != -1)
+	{
 		markasmissed(kanjiP);
 	}
 
@@ -382,12 +416,13 @@ void guessvalue(int guess){
 }
 
 
-/* printkanji:
+/* setkanjilabel:
  *	Takes widget, and translation struct.
  *	Sets widget label to be kanji string.
  *	Will set font.
  */
-void printkanji(Widget w,TRANSLATION trans){
+void setkanjilabel(Widget w,TRANSLATION trans)
+{
 
 	XtVaSetValues(w,	XtNencoding,XawTextEncodingChar2b,
 				XtNfont,largekfont,
@@ -399,12 +434,12 @@ void printkanji(Widget w,TRANSLATION trans){
 
 }
 
-/* printenglish
+/* setenglabel
  *	Given widget, and kanji index,
  *	Sets widget label to appropriate english.
  *	Will change fonts to englishfont.
  */
-void printenglish(Widget widget,TRANSLATION trans)
+void setenglabel(Widget widget,TRANSLATION trans)
 {
 
 	XtVaSetValues(widget,
@@ -415,13 +450,14 @@ void printenglish(Widget widget,TRANSLATION trans)
 
 }
 
-/* printkana
+/* setkanalabel
  *	Given widget, and kanji index,
  *	Sets widget label to kana reading
  *	Will change font to smallkfont
  */
 
-void printkana(Widget widget,TRANSLATION trans){
+void setkanalabel(Widget widget,TRANSLATION trans)
+{
 	if(romajiswitch ==1)
 	{
 		char stringbuff[MAXROMAJI+1];
@@ -461,16 +497,18 @@ void printkana(Widget widget,TRANSLATION trans){
  *	prints out question button, in appropriate font.
  */
 
-void printquestion(){
-	switch(questionmode){
+void printquestion()
+{
+	switch(questionmode)
+	{
 		case GUESS_ENGLISH:
-			printenglish(questionWidget,values[truevalue]);
+			setenglabel(questionWidget,values[truevalue]);
 			break;
 		case GUESS_KANA:
-			printkana(questionWidget,values[truevalue]);
+			setkanalabel(questionWidget,values[truevalue]);
 			break;
 		case GUESS_KANJI:
-			printkanji(questionWidget,values[truevalue]);
+			setkanjilabel(questionWidget,values[truevalue]);
 			break;
 		default:
 			fprintf(stderr,"Internal Error in kdrill: printallchoices\n");
@@ -485,21 +523,24 @@ void printquestion(){
  *	updates all "choices" labels...
  *	have to handle english, kana, or kanji!
  */
-void printallchoices(){
+void printallchoices()
+{
 	int i;
 
-	for(i=0;i<NUMBEROFCHOICES;i++){
+	for(i=0;i<NUMBEROFCHOICES;i++)
+	{
 		SetBlackOnWhite(choicesWidgets[i]);
 
-		switch(choicesmode){
+		switch(choicesmode)
+		{
 			case GUESS_ENGLISH:
-				printenglish(choicesWidgets[i],values[i]);
+				setenglabel(choicesWidgets[i],values[i]);
 				break;
 			case GUESS_KANA:
-				printkana(choicesWidgets[i],values[i]);
+				setkanalabel(choicesWidgets[i],values[i]);
 				break;
 			case GUESS_KANJI:
-				printkanji(choicesWidgets[i],values[i]);
+				setkanjilabel(choicesWidgets[i],values[i]);
 				break;
 			default:
 				fprintf(stderr,"Internal Error in kdrill: printallchoices\n");
@@ -513,12 +554,11 @@ void printallchoices(){
 
 
 /* SetupGuess
- *		"MAIN LOOP" OF PROGRAM
+ *	
  ******************************************************
  *
- *	sets up question...
- *	
- *	then returns, presumably falling back to XtAppMainLoop()
+ * called by guessvalue(), (or init) once user has clicked the CORRECT answer.
+ *	sets up new question and triggers button label refresh
  *
  */
 void SetupGuess()
@@ -526,14 +566,18 @@ void SetupGuess()
 	Boolean doloop;
 	int valuecount;
 
+	StopTimeout();
+
 	lastpicked = values[truevalue];
 
 	/* first determine which of the buttons will be the "true" choice */
 	truevalue = lrand48() %NUMBEROFCHOICES;
 
 	/* Now randomly assign characters for each choice button */
-	for(valuecount=0;valuecount<NUMBEROFCHOICES;valuecount++){
-		if(valuecount == truevalue){
+	for(valuecount=0;valuecount<NUMBEROFCHOICES;valuecount++)
+	{
+		if(valuecount == truevalue)
+		{
 			values[valuecount] = picktruevalue();
 		} else {
 			values[valuecount] = pickkanji();
@@ -547,7 +591,7 @@ void SetupGuess()
 	 *    UNLESS current value is the "truevalue".
 	 *  We then do a full compare again if there was a conflict
 	 */
-	do{
+	do {
 		int startcount=0;
 		doloop = False;
 		do {
@@ -575,6 +619,6 @@ void SetupGuess()
 	if(values[truevalue]->incorrect>0)
 		setstatus("You missed this one before");
 
+	StartTimeout();
 }
 
-
