@@ -23,6 +23,7 @@
 #include "options.h"
 #include "searchwidgets.h"
 #include "widgets.h"
+#include "timeout.h"
 
 #include "icon_xbm"
 
@@ -36,10 +37,17 @@ char usefilename[100];
 Atom wm_message,delete_message;
 
 /*
- * We  must have these resources set!!
+ * This array has "fallback resources", in case
+ * the normal "KDrill" resource file is not present.
+ * Note that it is an either/or situation.
+ * If there is a KDrill app-resources file on the system,
+ * these fallback resources WILL NOT BE LOOKED AT !!!
+ *
+ *  If you add something to this list, consider adding it to
+ *  SaveConfig() as well!!
+ *  And dont forget that you have to use SetXtrmXXXX() routines
+ *  to have the values available for SaveConfig() also!!
  *
- * ( Note the similarity between this stuff and optionDescList[].
- *   coincidence? I think not :-)
  */
 static char *fallback[] = {
 	"KDrill.options_popupx:		0",
@@ -49,7 +57,7 @@ static char *fallback[] = {
 	"KDrill.search_popupy:		0",
 	"KDrill.notallT:	0",
 	"KDrill.noBell:		0",
-	"KDrill.nousefile:	0",
+	"KDrill.nousefile:	1",
 	"KDrill.usefile:	.kanjiusefile", 
 	DICTLOCATION,	/* change this value in Imakefile !!*/
 	EDICTLOCATION,	/* change this value in Imakefile !!*/
@@ -57,11 +65,12 @@ static char *fallback[] = {
 	"KDrill.smallkanji:	-jis-*--16*jisx0208*",
 	"KDrill.englishfont:	fixed",
 	"KDrill.gradelevel:	0",
-	"KDrill.guessmeaning:	0",
-	"KDrill.showkana:	0",
+	"KDrill.guessmode:	english",
+	"KDrill.questionmode:	kanji",
 	"KDrill.romajiswitch:	0",
 	"KDrill.lowfrequency:	0",
 	"KDrill.highfrequency:	0",
+	"KDrill.timersec:	7",
 	"KDrill.logfile:	kdrill.log",
 	NULL
 };
@@ -156,34 +165,37 @@ void initgc(){
  *	(The nice thing is that thanks to Xt, commandline options
  *	 get auto-translated to resources already
  */
+
 void GetOptions(){
-	char tempnumber[10];
+	int tmpnumber;
 	char tmpstr[100];
 
 
 	/* set grade bits appropriately */
-	GetXtString("gradelevel","Gradelevel",tempnumber);
-	parsegrades(tempnumber);
+	GetXtrmString("gradelevel","Gradelevel",tmpstr);
+	parsegrades(tmpstr);
 
-	switchKanaEnglish = !GetXtBoolean("notallT","NotallT");
+	switchKanaEnglish = !GetXtrmBoolean("notallT","NotallT");
 	/* default of nousefile is FALSE. which means
 	 * useUsefile defaults to TRUE
 	 */
-	useUsefile = !GetXtBoolean("nousefile","Nousefile");
+	useUsefile = !GetXtrmBoolean("nousefile","Nousefile");
 
-	doBell = !GetXtBoolean("noBell","NoBell");
-	showinorder = GetXtBoolean("showinorder","Showinorder");
-	if(GetXtBoolean("guessmeaning","Guessmeaning"))
-		choicesmode = GUESS_ENGLISH;
+	doBell = !GetXtrmBoolean("noBell","NoBell");
+	showinorder = GetXtrmBoolean("showinorder","Showinorder");
 
-	lowfrequency = GetXtNumber("lowfrequency","Lowfrequency");
+	lowfrequency = GetXtrmNumber("lowfrequency","Lowfrequency");
 	if((lowfrequency <0) || (lowfrequency >=MAXTRANSLATIONSALLOWED))
 		lowfrequency = 0;
-	highfrequency = GetXtNumber("highfrequency","Highfrequency");
+	highfrequency = GetXtrmNumber("highfrequency","Highfrequency");
 	if((highfrequency <0) || (highfrequency >=MAXTRANSLATIONSALLOWED))
 		highfrequency = 0;
 
-	GetXtString("guessmode", "Guessmode", tmpstr);
+	tmpnumber = GetXtrmNumber("timersec","Timersec");
+	setTimeoutLen(tmpnumber);
+	
+
+	GetXtrmString("guessmode", "Guessmode", tmpstr);
 	if(strcmp(tmpstr, "english")==0){
 		choicesmode = GUESS_ENGLISH;
 	}else if(strcmp(tmpstr, "kanji")==0){
@@ -191,7 +203,7 @@ void GetOptions(){
 	} else{
 		choicesmode = GUESS_KANA;
 	}
-	GetXtString("questionmode", "Questionmode", tmpstr);
+	GetXtrmString("questionmode", "Questionmode", tmpstr);
 	if(strcmp(tmpstr, "english")==0){
 		questionmode=GUESS_ENGLISH;
 	} else if (strcmp(tmpstr, "kanji")==0){
@@ -200,7 +212,7 @@ void GetOptions(){
 		questionmode=GUESS_KANA;
 	}
 
-	if( GetXtBoolean("romajiswitch","Romajiswitch"))
+	if( GetXtrmBoolean("romajiswitch","Romajiswitch"))
 		romajiswitch = 1;
 
 }
@@ -257,7 +269,7 @@ void setup_deletewindow(Widget winwidget)
 void init_xdb()
 {
 	char XFileName[_POSIX_PATH_MAX];
-	XrmDatabase xdb, globaldb;
+	XrmDatabase globaldb;
 
 	sprintf(XFileName,"%s/.kdrill", homedir);
 	globaldb=XrmGetDatabase(display);
