@@ -4,7 +4,9 @@
 
 extern void MakeMulti();
 extern void ClearAllMulti();
-extern void AddTranslation(TRANSLATION);
+extern void AddMultiTranslation(TRANSLATION);
 extern int getMultiMax();
+extern int getMultiCount();
 
 extern void ShowMulti();
+extern void setmultilabels();
