@@ -106,16 +106,19 @@ void StripBrackets(char *dest,unsigned char *source)
 }
 /* StripSlash
  *	Gets rid of /enlish/english2/ Slashes.
+ *	Copies the cleaned up version of source, to topdest
+ *
  *	This is for readedict. Probably nothing else should use it
  *	Modeled directly after StripBrackets
  *	PRESUMES first char of source is '/'!!
  *	Then looks for LAST '/'
- *	(Or will set dest[]0] to '\0')
+ *	(Or will set topdest[0] to '\0')
  *
- *	we USED to translate middle ':' to '/'.
+ *	We USED to translate middle ':' to '/'.
  *
  *	Source is actually assumed to be regular ascii signed char,
  *	but declared as unsigned to stop compiler warnings.
+ *
  * return 0 OKAY, 1 bad line
  */
 int StripSlash(char *topdest,unsigned char *source)
@@ -125,7 +128,7 @@ int StripSlash(char *topdest,unsigned char *source)
 	unsigned char *parse = source;
 
 	if(*parse != '/'){
-		dest[0] = '\0';
+		topdest = '\0';
 		return 1;
 	}
 	parse=strrchr(source, '/');
@@ -138,6 +141,9 @@ int StripSlash(char *topdest,unsigned char *source)
 	strncpy(topdest, &source[1], parse- source - 1);
 	topdest[englen]='\0';
 
+	/* we've copied the relavant part over to topdest. Now rewrite
+	 * in-place
+	 */
 	dest=topdest;
 	dest=strchr(dest, '/');
 	while(dest!=NULL){
@@ -424,14 +430,19 @@ void readedict()
 	/* the following will be NULL if kanjidic not read in */
 	lastk = translations[highestkanji];
 
-	GetXtString("edictfile","Edictfile",edict);
+	GetXtrmString("edictfile","Edictfile",edict);
 	edictname = edict;
 
-	fp = open_compressed(edictname);
+	if(strncmp(edictname,"none",4)==0){
+		fprintf(stderr,"edictfile set to 'none'. Skippping.\n");
+		return;
+	}
 
+	fp = open_compressed(edictname);
 	if(fp == NULL)
 	{
-		fprintf(stderr,"Cannot open edict file. Skipping.\n");
+		fprintf(stderr,"Cannot open edict file %s. Skipping.\n",
+			edictname);
 		return;
 	}
 	printf("Opened dictionary %s \n",edictname);
@@ -573,7 +584,6 @@ short skipfromthree(int one, int two, int three){
 short parseskip(char *input)
 {
 	int one, two, three;
-	int SKIPnum;
 	
 	one = atoi(input);
 
@@ -617,16 +627,21 @@ void readstructs(){
 	FILE *fp;
 	TRANSLATION newk=NULL,lastk=NULL;
 
-	GetXtString("kdictfile","Kdictfile",dict);
+	GetXtrmString("kdictfile","Kdictfile",dict);
 	dictname = dict;
 #ifdef DEBUG
 	printf("kdictfile from resources is\" %s\"\n",dictname);
 #endif
 
+	if(strncmp(dictname,"none",4)==0){
+		fprintf(stderr,"kdictfile set to 'none'. Skippping.\n");
+		return;
+	}
 	fp = open_compressed(dictname);
 	if(fp == NULL)
 	{
-		fprintf(stderr,"Cannot open kanjidic file. Skipping.\n");
+		fprintf(stderr,"Cannot open kanjidic file %s. Skipping.\n",
+			dictname);
 		return;
 	}
 	
@@ -669,7 +684,7 @@ void readstructs(){
 			continue;
 		}
 		/* now parse for grade level, frequency, and english */
-		freq = grade = N = U = H = 0;
+		freq = grade = N = U = H = SKIP=0;
 		strokes=0; Q = -1; /* remember, "0000" IS a valid Qval!*/
 
 		nextword(&parse);
