@@ -7,6 +7,7 @@
 #include <Xos.h>
 #include <Intrinsic.h>
 #include <StringDefs.h>
+#include <Xfuncs.h>
 
 #include <Shell.h>
 
@@ -28,10 +29,15 @@
 
 #include "searchwidgets.h"
 
+#include "utils.h"
+#include "init.h"
+#include "log.h"
+
 
 Cursor textcursor;
 
 Widget search_popup=NULL;
+Widget searchstatusline=NULL;
 
 Widget searchwidgets[NUM_OF_W];
 Widget searchnumbers[NUM_OF_N];
@@ -45,7 +51,7 @@ Widget searchkanjiW,searchokuW; /* display current kanji here */
 XChar2b kanastring[MAXKANALENGTH]={{0,0}};
 
 /* flag, if we only match kanji/kana at beginning of translation */
-int kana_onlyatstart=0;
+int match_onlyatstart=0;
 
 
 static char *searchAccel = 
@@ -53,39 +59,6 @@ static char *searchAccel =
 
 
 
-/* These two could be more general case utilities, but for now are only
- * used here.
- * We take a tip from java, and use the PARENT to determine background
- * and foreground
- *
- * UNFORTUNATELY... foreground never seems to work. so we'll fake
- * it with black. sigh.
- */
-void ReverseButton(Widget button)
-{
-	Pixel fg, bg;
-	XtVaGetValues(XtParent(button), XtNforeground, &fg,NULL);
-	XtVaGetValues(XtParent(button), XtNbackground, &bg, NULL);
-
-	fg=black;
-
-	XtVaSetValues(button, XtNforeground, bg,
-		      XtNbackground, fg,
-		      NULL);
-}
-void UnreverseButton(Widget button)
-{
-	Pixel fg, bg;
-	XtVaGetValues(XtParent(button), XtNforeground, &fg,NULL);
-	XtVaGetValues(XtParent(button), XtNbackground, &bg, NULL);
-
-	fg=black;
-
-	XtVaSetValues(button, XtNforeground, fg,
-		      XtNbackground, bg,
-		      NULL);
-}
-
 
 
 /* This gets called after clicking on the "search" button, or
@@ -93,7 +66,7 @@ void UnreverseButton(Widget button)
  */
 void SearchCallback(Widget w,XtPointer client_data, XtPointer call_data)
 {
-	static search_up = -1; 
+	static int search_up = -1; 
 	static Position rel_x,rel_y;
 	Position x,y;
 
@@ -101,8 +74,8 @@ void SearchCallback(Widget w,XtPointer client_data, XtPointer call_data)
 
 	if(search_up==-1){
 		/* first time init.. */
-		rel_x = GetXtNumber("search_popupx","Search_popupx");
-		rel_y = GetXtNumber("search_popupy","Search_popupy");
+		rel_x = GetXtrmNumber("search_popupx","Search_popupx");
+		rel_y = GetXtrmNumber("search_popupy","Search_popupy");
 		setup_deletewindow(search_popup);
 		search_up=0;
 	}
@@ -134,7 +107,7 @@ void SearchCallback(Widget w,XtPointer client_data, XtPointer call_data)
 void Handle_starttoggle(Widget w,XtPointer client_data, XtPointer call_data)
 {
 	Pixel fg,bg;
-	kana_onlyatstart=!kana_onlyatstart;
+	match_onlyatstart=!match_onlyatstart;
 
 	XtVaGetValues(w, XtNforeground, &fg,
 		      XtNbackground, &bg, NULL);
@@ -144,6 +117,13 @@ void Handle_starttoggle(Widget w,XtPointer client_data, XtPointer call_data)
 		    
 }
 
+/* deal with matchkanji button. */
+/* Find all matches for what is currently being displayed by the search win*/
+void Handle_matchkanji(Widget w,XtPointer client_data, XtPointer call_data)
+{
+	findkanjiall(lastsearch->kanji);
+}
+
 /* Given a translation, return the index into translations[] that it
  * sits at
  */
@@ -163,7 +143,7 @@ int trans_to_index(TRANSLATION trans)
 	return kcount;
 }
 
-/* handle toggle for "this kanji in in usefile"*/
+/* handle toggle for "This kanji in in usefile" button*/
 void Handle_usetoggle(Widget w,XtPointer client_data, XtPointer call_data)
 {
 	Pixel fg,bg;
@@ -179,8 +159,16 @@ void Handle_usetoggle(Widget w,XtPointer client_data, XtPointer call_data)
 		    
 }
 
+/* handle "Show" usefile button */
+void Handle_showusefile(Widget w,XtPointer client_data, XtPointer call_data)
+{
+	setstatus("Showing usefile...");
+
+	dousefilefind();
+}
 
 /* Update the labels for the info widgets at the top of search window.*/
+/* Assumes variable "lastsearch" points to currently desired TRANSLATION */
 void updatesearchlabels()
 {
 	TRANSLATION This=lastsearch;
@@ -323,7 +311,7 @@ long TextLength (w)
  * We don't do anything in this routine any more.
  * but its nice to keep around for debug purposes.
  */
-doFunkyGrab(Widget textwin)
+void doFunkyGrab(Widget textwin)
 {
 	int length; /* this is nastynastynasty...*/
 	XPointer value;
@@ -332,6 +320,7 @@ doFunkyGrab(Widget textwin)
 
 	length=TextLength(textwin);
 
+	/* Okay, this is pseudo-undocumented. sorry. its what works! */
 	value = (XPointer)_XawTextGetSTRING((TextWidget) textwin, 0, length);
 	if(value==NULL)
 	{
@@ -367,7 +356,6 @@ void Handle_searchPasted(Widget widget, XtPointer cl, XEvent *e, Boolean *cont)
 	XChar2b kstring[4];
 	String str;
 	int strparse;
-	int searchchar;
 	
 	if(e->type != ButtonRelease)
 		return;
@@ -395,11 +383,25 @@ void Handle_searchPasted(Widget widget, XtPointer cl, XEvent *e, Boolean *cont)
 #endif
 
 	if(strparse<2)
+	{
+		puts("Paste error: Less than 2 bytes pasted");
 		return;
+	}
+
+	/* POSSIBILITIES:
+	 * 1. "raw" JIS, that looks to us like
+	 *   0xffffff##, 0xffffff##
+	 *
+	 * 2. prefaced JIS, that looks to us like
+	 *   0x1b, 0x2d, 0x41, 0xffffff##, 0xffffff##
+	 * 
+	 * 3. A different (older?) kind of prefaced JIS,that looks like
+	 *   0x1b, 0x24, 0x29, 0x42, blahblahblah
+	 */
 
 	if(str[0]&0xffffff00)
 	{
-		puts("DEBUG: pure JIS");
+		puts("DEBUG: pure JIS paste");
 
 		/* just PURE JIS (probably) */
 		kstring[0].byte1=str[0]&0x7f;
@@ -410,37 +412,39 @@ void Handle_searchPasted(Widget widget, XtPointer cl, XEvent *e, Boolean *cont)
 			kstring[1].byte1=str[2]&0x7f;
 			kstring[1].byte2=str[3]&0x7f;
 		}
+		if(str[4]&0xffffff00)
+		{
+			kstring[2].byte1=str[4]&0x7f;
+			kstring[2].byte2=str[5]&0x7f;
+		}
 
 	}
-	else
-	{
-		/* otherwise, assume iso2022 or whatever-encoded */
-		/* leading 1b242942 is expected */
-	
-		if(strparse <7 )
-		{
-			puts("paste too short. bye-bye");
-			XtVaSetValues(widget,XtNstring,"\0",NULL);
+	else if (str[0] == 0x1b){
+		int startchar=0;
+		/* do goodstuff.. figure out which of the two it is */
+		if((str[1]==0x2d) &&
+		   (str[2]==0x41)){
+			startchar=3;
+		}
+		else if((str[1]==0x24) &&
+			(str[2]==0x29) &&
+			(str[3]==0x42)){
+			startchar=4;
+		}else{
+			puts("DEBUG: unrecognized paste encoding");
 			return;
 		}
-		if(str[0] != 0x1b)
+		
+		kstring[0].byte1=str[startchar]&0x7f;
+		kstring[0].byte2=str[startchar+1]&0x7f;
+		if(str[startchar+2] != 0)
 		{
-			puts("oops. cannot recognise encoding");
-			printf("starts with: x%x x%x x%x x%x\n",
-			       str[0],str[1],str[2]);
-			XtVaSetValues(widget,XtNstring,"\0",NULL);
-			return;
-		}
-		kstring[0].byte1=str[4]&0x7f;
-		kstring[0].byte2=str[5]&0x7f;
-		if(str[6] != 0)
-		{
-			kstring[1].byte1=str[6]&0x7f;
-			kstring[1].byte2=str[7]&0x7f;
-			if(str[8] != 0)
+			kstring[1].byte1=str[startchar+2]&0x7f;
+			kstring[1].byte2=str[startchar+3]&0x7f;
+			if(str[startchar+4] != 0)
 			{
-				kstring[2].byte1=str[8]&0x7f;
-				kstring[2].byte2=str[9]&0x7f;
+				kstring[2].byte1=str[startchar+4]&0x7f;
+				kstring[2].byte2=str[startchar+5]&0x7f;
 			}
 		}
 	}
@@ -471,13 +475,12 @@ void makenumbers(Widget parent){
 		       NULL);
 	searchnumbers[H_INPUT] = XtVaCreateWidget("7",asciiTextWidgetClass,parent,
 		       XtNfromVert,searchnumbers[G_LABEL],
-		       XtNfromHoriz,searchnumbers[H_LABEL],
+		       XtNfromHoriz,searchnumbers[G_LABEL],
 		       XtNstring," ",  XtNwidth,INPUTWIDTH,
 		       XtNeditType,XawtextEdit,
 		       NULL);
 
 	searchnumbers[F_LABEL] = XtVaCreateWidget("2",labelWidgetClass,parent,
-		       XtNfromHoriz,searchnumbers[G_INPUT],
 		       XtNfromHoriz,searchnumbers[H_INPUT],
 		       XtNlabel,"F:",  XtNborderWidth,0,
 		       NULL);
@@ -488,13 +491,12 @@ void makenumbers(Widget parent){
 		       NULL);
 	searchnumbers[N_LABEL] = XtVaCreateWidget("8",labelWidgetClass,parent,
 		       XtNfromVert,searchnumbers[G_LABEL],
-		       XtNfromHoriz,searchnumbers[G_INPUT],
 		       XtNfromHoriz,searchnumbers[H_INPUT],
 		       XtNlabel,"N:",   XtNborderWidth,0,
 		       NULL);
 	searchnumbers[N_INPUT] = XtVaCreateWidget("9",asciiTextWidgetClass,parent,
 		       XtNfromVert,searchnumbers[G_LABEL],
-		       XtNfromHoriz,searchnumbers[N_LABEL],
+		       XtNfromHoriz,searchnumbers[F_LABEL],
 		       XtNstring," ",  XtNwidth,INPUTWIDTH,
 		       XtNeditType,XawtextEdit,
 		       NULL);
@@ -527,6 +529,16 @@ void makenumbers(Widget parent){
 			"12",commandWidgetClass,parent,
 			XtNlabel, "usefile",
 			XtNfromHoriz,searchnumbers[U_INPUT],
+			XtNhorizDistance, 20,
+		       NULL);
+	searchnumbers[SHOWUSEFILE] = XtVaCreateWidget(
+			"12",commandWidgetClass,parent,
+			XtNlabel, "show u.",
+			XtNshapeStyle,XawShapeRoundedRectangle,
+			XtNcornerRoundPercent,50,
+			XtNfromHoriz,searchnumbers[U_INPUT],
+			XtNfromVert,searchnumbers[INUSEFILE],
+			XtNhorizDistance, 20,
 		       NULL);
 
 	XtManageChildren(searchnumbers,NUM_OF_N);
@@ -537,7 +549,7 @@ void makenumbers(Widget parent){
  * Make widgets dealing with "current kanji", at top of search popup
  */
 void maketopsearchstuff(Widget parent){
-	Widget numbersform;
+	Widget numbersform,matchbutton;
 
 	searchkanjiW = XtVaCreateManagedWidget("kanjilarge",
 						  labelWidgetClass,
@@ -548,10 +560,16 @@ void maketopsearchstuff(Widget parent){
 			XtNencoding,XawTextEncodingChar2b,
 			XtNfont,largekfont,
 			NULL);
+	matchbutton = XtVaCreateManagedWidget("matchkanjib",
+			commandWidgetClass,parent,
+			XtNfromHoriz,searchkanjiW,
+			XtNlabel, "match",
+			NULL);
 	
+	XtAddCallback(matchbutton, XtNcallback, Handle_matchkanji, NULL);
+
 	numbersform = XtVaCreateManagedWidget("numform",
-					formWidgetClass,
-					parent,
+			formWidgetClass, parent,
 			XtNfromVert,searchkanjiW,
        			NULL);
 
@@ -656,41 +674,36 @@ void makesearchinputs(Widget parent)
 	/* and now the kana search input widgets. gets a bit messy here */
 
 	searchwidgets[KANA_FORM]=XtVaCreateManagedWidget("kanasearchgroup",
-			 formWidgetClass,
-			 parent,
+			 formWidgetClass, parent,
 			 XtNfromVert, kanjisform,
 			 XtNvertDistance, 10,
 			 XtNborderWidth, 1,
 			 NULL);
 
 
+	searchwidgets[SEARCH_STARTTOG]=
+		XtVaCreateManagedWidget("atstart_toggle",
+			commandWidgetClass, parent,
+			XtNfromVert, kanjisform,
+			XtNvertDistance, 10,
+			XtNfromHoriz, searchwidgets[KANA_FORM],
+			XtNhorizDistance, 15,
+			XtNlabel, "[...",
+			NULL);
+		
 
 	searchwidgets[SEARCH_KANA_L] =
 		XtVaCreateManagedWidget("searchkanalabel",
-			commandWidgetClass,
-			searchwidgets[KANA_FORM],
+			commandWidgetClass, searchwidgets[KANA_FORM],
 			XtNjustify, XtJustifyRight,
 			XtNlabel, "Kana search   ",
 			/*XtNborderWidth, 0,*/
 			NULL);
 
-	/* Difference here */
 
-	searchwidgets[SEARCH_STARTTOG]=
-		XtVaCreateManagedWidget("atstart_toggle",
-			commandWidgetClass,
-			searchwidgets[KANA_FORM],
-			XtNfromHoriz, searchwidgets[SEARCH_KANA_L],
-			XtNhorizDistance, 30,
-			XtNlabel, "[...",
-			NULL);
-		
-		/*0x214e,0x2144*/
-						       
 	searchwidgets[SEARCH_KANA_W] =
 		XtVaCreateManagedWidget("searchkana",
-			labelWidgetClass,
-			searchwidgets[KANA_FORM], 
+			labelWidgetClass, searchwidgets[KANA_FORM], 
 			XtNwidth, FULLWIDTH - 100,
 			XtNlabel,"\0",
 			/*XtNlabel, "!z",*/ /* 0x217a */
@@ -716,8 +729,11 @@ void makesearchinputs(Widget parent)
 
 	XtAddCallback(searchwidgets[SEARCH_STARTTOG], 
 		      XtNcallback, Handle_starttoggle, NULL);
+
 	XtAddCallback(searchnumbers[INUSEFILE], 
 		      XtNcallback, Handle_usetoggle, NULL);
+	XtAddCallback(searchnumbers[SHOWUSEFILE], 
+		      XtNcallback, Handle_showusefile, NULL);
 
 
 
@@ -793,6 +809,15 @@ void MakeSearchPopup()
 	 */
 	makesearchinputs(searchform);
 
+	searchstatusline = XtVaCreateManagedWidget("searchstatus",
+			labelWidgetClass, searchform,
+		      XtNlabel,"Search popup",
+		      XtNwidth,FULLWIDTH,
+		      XtNborderWidth,2,
+		      XtNfromVert, searchwidgets[KANA_FORM],
+		      XtNresize,True,
+		      NULL);
+
 	Accel = XtParseAcceleratorTable(searchAccel);
 	XtOverrideTranslations(searchwidgets[SEARCH_ENG_W],Accel);
 
