@@ -6,6 +6,8 @@
 #include <stdio.h>
 
 #include <Intrinsic.h>
+#include <IntrinsicP.h>
+
 #include <StringDefs.h>
 #include <Xfuncs.h>
 
@@ -75,7 +77,9 @@ void Beep(){
 /* Since there is no standard util to convert hex ascii to int,...
  *  have to supply our own..
  *  It isn't incredibly efficient.. let's hope the compiler is smart.
- *  only used in readstructs()
+ *
+ * Note that we take char* as a hex string, EVEN IF it does not
+ * start with 0x
  */
 int xtoi(char * s)
 {
@@ -212,98 +216,116 @@ TRANSLATION FindField(TRANSLATION startK,
 
 
 
+/* GetWidgetNumberval
+ *	Get the asciiText from an input widget.
+ *	Attempt to look up kanji index
+ *	Set asciiText label to null if not found, or
+ *	a clean decimal print of index otherwise.
+ *
+ *	Calls FindIndex, which accepts special prefixes.
+ */
+int GetWidgetNumberval(Widget w){
+	String str;
+	int retval=0;
+
+
+	XtVaGetValues(w,XtNstring,&str,NULL);
+	retval=atoi(str);
+	SetWidgetNumberval(w,retval);
 
 /*
- * FindIndex:
- *
- *  Given ascii string represending index, find actual JIS index.
- *
- *	Has to handle special prefixes:
- *
- *	0x == hex index
- *	U  == Unicode index		(in HEX )
- *	N  == Nelson dictionary index	(in DEC )
- *	H  == Halpern dictionary index    (in DEC )
- *
- *	Should set user status message on faliure
- */
-
-int FindIndex(char *str){
-	char statusmsg[100];  /* This gets seeded with potential
-				* error messages, even if error does not
-				* happen
-				*/
-				
-	int retval=0;
-	int tempval,findit;
-
-
-	sprintf(statusmsg,"Could not find index");
-
-
-	switch (str[0]){
-		case '0':
-			if(strncmp(str,"0x",2) == 0){
-				sprintf(statusmsg,"Could not find hexadecimal index");
-				retval = xtoi(&str[2]);
-			}
-			else
-				retval = atoi(str);
- 			break;
-		case 'H':
-			sprintf(statusmsg,"Could not find H index");
-			tempval = atoi(&str[1]);
-			retval = 0;
-			if(tempval == 0)
-				break;
-			for(findit =lowestkanji;findit<highestkanji;findit++){
-				if(translations[findit] == NULL)
-					continue;
-				if(translations[findit]->Hindex == tempval){
-					retval = findit;
-					break;
-				}
-			}
-			break;
-		case 'N':
-			sprintf(statusmsg,"Could not find N index");
-			tempval = atoi(&str[1]);
-			retval = 0;
-			if(tempval == 0)
-				break;
-			for(findit =lowestkanji;findit<highestkanji;findit++){
-				if(translations[findit] == NULL)
-					continue;
-				if(translations[findit]->Nindex == tempval){
-					retval = findit;
-					break;
-				}
-			}
-			break;
-		case 'U':
-			sprintf(statusmsg,"Could not find U index");
-			tempval = xtoi(&str[1]);
-			retval = 0;
-			if(tempval == 0)
-				break;
-			for(findit =lowestkanji;findit<highestkanji;findit++){
-				if(translations[findit] == NULL)
-					continue;
-				if(translations[findit]->Uindex == tempval){
-					retval = findit;
-					break;
-				}
-			}
-			break;
-
-		default:
-			retval = atoi(str);
-			break;
+	if(w == currentkanjiNum){
+		retval = xtoi(str);
+		SetWidgetHexval(w,retval);
+	} else {
+		retval = FindIndex(str);
+		SetWidgetNumberval(w,retval);
 	}
-	if(retval == 0){
-		setstatus(statusmsg);
-		Beep();
-	}
-
-	return retval;
+*/
+	return (retval);
 }
+
+/* GetWidgetHexval
+ *	Like GetWidgetNumberval(), except assumes that
+ *	the widget value MUST BE A HEX STRING
+ *	Get the asciiText from an input widget.
+ *	Set asciiText label to null if not found, or
+ *	a clean print of index otherwise.
+ *
+ *	Calls FindIndex, which accepts special prefixes.
+ */
+int GetWidgetHexval(Widget w){
+	String str;
+	int retval=0;
+
+
+	XtVaGetValues(w,XtNstring,&str,NULL);
+	retval = xtoi(str);
+	SetWidgetHexval(w,retval);
+
+	return (retval);
+}
+
+int GetWidgetWidth(Widget w){
+	Dimension wwidth;
+	XtVaGetValues(w, XtNwidth, &wwidth, NULL);
+	return wwidth;
+}
+int GetWidgetBWidth(Widget w){
+	Dimension wwidth;
+	XtVaGetValues(w, XtNborderWidth, &wwidth, NULL);
+	return wwidth;
+}int GetWidgetHeight(Widget w){
+	Dimension wheight;
+	XtVaGetValues(w, XtNheight, &wheight, NULL);
+	return wheight;
+}
+
+/* Note that this sets borderwidth to 0!! */
+void WidenWidget(Widget w, int newwidth){
+	int height, bwidth;
+
+	height=GetWidgetHeight(w);
+	bwidth=GetWidgetBWidth(w);
+	
+#ifdef NONO
+	/*XtResizeWidget(w, newwidth, height, bwidth);*/
+#else
+	printf("Resizing some window to width %d\n",newwidth);
+	XtVaSetValues(w,XtNwidth,newwidth,NULL);
+	XtResizeWindow(w);
+#endif
+
+}
+
+/* We take a tip from java, and use the PARENT to determine background
+ * and foreground
+ *
+ * UNFORTUNATELY... foreground never seems to work. so we'll fake
+ * it with black. sigh.
+ */
+void ReverseButton(Widget button)
+{
+	Pixel fg, bg;
+	XtVaGetValues(XtParent(button), XtNforeground, &fg,NULL);
+	XtVaGetValues(XtParent(button), XtNbackground, &bg, NULL);
+
+	fg=black;
+
+	XtVaSetValues(button, XtNforeground, bg,
+		      XtNbackground, fg,
+		      NULL);
+}
+void UnreverseButton(Widget button)
+{
+	Pixel fg, bg;
+	XtVaGetValues(XtParent(button), XtNforeground, &fg,NULL);
+	XtVaGetValues(XtParent(button), XtNbackground, &bg, NULL);
+
+	fg=black;
+
+	XtVaSetValues(button, XtNforeground, fg,
+		      XtNbackground, bg,
+		      NULL);
+}
+
