@@ -12,3 +12,10 @@ int		FindIndex(char *);
 int		isapipe(FILE *);
 
 Boolean isMapped(Widget w);
+
+extern void ReverseButton(Widget);
+extern void UnreverseButton(Widget);
+
+extern int GetWidgetNumberval(Widget);
+extern int GetWidgetHexval(Widget);
+extern int GetWidgetWidth(Widget);
